/*--------------------------------------------------------
  "THE BEER-WARE LICENSE" (Revision 42):
  Alex Kostyuk wrote this code. As long as you retain this 
  notice you can do whatever you want with this stuff. 
  If we meet some day, and you think this stuff is worth it, 
  you can buy me a beer in return.
----------------------------------------------------------*/ 

#include "includes.h"

int loop = 1, keypress;
SAMPLE *the_sample = NULL;
float scale_factor;
HDC hdc;
FILE *inifile;
BITMAP *scrat, *baro, *alt,*feet,*_1K,*_10K,*frame;
float hdg = 0;
float baro_val=29.92, feet_val=0,_1K_val=0,_10K_val=0;
int alt_val;
int alt_ap = 0;
int alt_ap_prev = 0;
int alert_flag = 0;
int alert_timer = 0;


int initialise_window( void );
int connect_to_sim( void );
int update_display( void );
int create_bitmaps( void );
int act_on_keypress( void );
int close_program( void );
void exit_hook (void );

#define ALERT_DELTA  500
#define TRIGGER_DELAY 20

void alt_alert_periodic(void)
{
    int delta = (alt_ap - alt_val);

    if(alt_ap != alt_ap_prev)
    {
        if(console_flag)
            printf("*");
        alert_timer = TRIGGER_DELAY;
        alert_flag = 1;
    }
    else
    {
        if(alert_timer)
        {
            if(console_flag)
                printf(".");
            if(--alert_timer == 0)
            {
                alert_flag = 0;
                if(console_flag)
                    printf(" triggered ");
            }

        }
    }
    alt_ap_prev = alt_ap;

    if(delta < 0)
        delta *= -1;
        if(console_flag)
            printf(" %d-%d ",delta,alert_flag);
    if(delta < ALERT_DELTA && delta > (ALERT_DELTA/10) && alert_flag == 0)
    {
        alert_flag = 1;
        if(console_flag)
            printf(" ALERT ");
        play_sample(the_sample,128, 128, 1000, 0);
    }

}


int main(int argc, char *argv[] )
{

    STARTUPINFO cif;
    PROCESS_INFORMATION pi;
    load_config();


    if(0 == strcmp(config.console,"off") && argc == 1)
    {
        ZeroMemory(&cif,sizeof(STARTUPINFO));
        if(CreateProcess(NULL,"altfeet.exe AS_DAEMON",
                         NULL,NULL,FALSE,CREATE_NO_WINDOW,NULL,NULL,&cif,&pi))

        {
            CloseHandle( pi.hProcess );
            CloseHandle( pi.hThread );
            exit(0);
        }
    }

    if(config.uplink_port)
        server_start(config.uplink_port);


    if(config.downlink_port)
    {
        udp_sender_start(config.downlink_port, config.downlink_ip);
    }


    initialise_window();

    /* install a digital sound driver */
    if (install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL) != 0)
    {
        allegro_message("Error initialising sound system\n%s\n", allegro_error);
        return 1;
    }

    /* read in the WAV file */
    the_sample = load_wav("..\\Resources\\alti\\alert.wav");
    if (!the_sample)
    {
        allegro_message("Error reading WAV file '%s'\n", argv[1]);
        return 1;
    }

    if( create_bitmaps() == -1 )
    {
        textprintf_centre(screen, font, config.size / 2, config.size * 0.15, 65535,  "*** Error ***" );
        textprintf_centre(screen, font, config.size / 2, config.size * 0.40, 65535,  "No Graphics files" );
        textprintf_centre(screen, font, config.size / 2, config.size * 0.65, 65535,  "Press Key to Exit" );
        readkey();
        close_program();
        return( -1 );
    }

    update_display();

    while( loop == 1 )
    {

        while( keypressed() )
        {
            act_on_keypress();
            update_display();
        }
        clear_keybuf();
        update_display();
    }
    destroy_sample(the_sample);
    save_config();
    close_program();
    return (0);
}
END_OF_MAIN()

int initialise_window( void )
{
    hdc = GetDC(NULL);
    int screen_colour_depth = GetDeviceCaps (hdc, BITSPIXEL );

    if (allegro_init() != 0) return 1; //Initialise Allegro
    install_keyboard();

    set_color_depth(screen_colour_depth);


    if (set_gfx_mode(GFX_AUTODETECT_WINDOWED, 800, 600, 0, 0) != 0)
    {
        if (set_gfx_mode(GFX_SAFE, 800, 600, 0, 0) != 0)
        {
            set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
            allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
            return 1;
        }
    }

    SetWindowPos(win_get_window(), HWND_TOPMOST,
                 config.x_pos,
                 config.y_pos,
                 config.size+6,
                 config.size+25,
                 SWP_ASYNCWINDOWPOS);

    set_display_switch_mode(SWITCH_BACKGROUND);
    scale_factor = ((float)config.size / 400);

    set_window_close_hook(exit_hook);
    return( 0 );
}



int update_display( void )

{
    int size = config.size;


    pivot_scaled_sprite(scrat, baro, size / 2, (size / 2),256, 256,

                        ftofix(

                            (   (29.92-baro_val)*  248.96373057   +519.0)  /3.90625

                        ) ,

                        ftofix(scale_factor*0.73));



    rotate_scaled_sprite(scrat, alt, 00, 00, ftofix(0) , ftofix(scale_factor*0.5));
    rotate_scaled_sprite(scrat, _10K, 00, 00, ftofix((alt_val/100)/3.90625) , ftofix(scale_factor));
    pivot_scaled_sprite(scrat, _1K, size / 2, (size / 2),100, 396,
                        ftofix((alt_val/10)/3.90625) , ftofix(scale_factor*0.5));
    pivot_scaled_sprite(scrat, feet
                        , size / 2, (size / 2),340, 340, ftofix(((alt_val%1000)*1.0)/3.90625) , ftofix(scale_factor*0.5));


    rotate_scaled_sprite(scrat, frame, 00, 00, ftofix(0) , ftofix(scale_factor*0.666666666));



    alt_alert_periodic();

    textprintf_ex(scrat, font, 5, 5, makecol(255, 255, 000),-1,
                  "%5d FT",alt_val
                 );

    textprintf_ex(scrat, font, 5, size-15, makecol(255, 255, 000),-1,
                  "%4.1f hPa",baro_val*33.86
                 );



    textprintf_ex(scrat, font, size-70, 5, makecol(000, 255, 000),-1,
                  "%5.0f M",
                  (alt_val*1.0)/3.28084
                 );

    textprintf_ex(scrat, font, size-70, size-15, makecol(000, 255, 000),-1,
                  "%3.1f mm",
                  baro_val * 25.4
                 );


    blit(scrat, screen, 0, 0, 0, 0, size, size);
    Sleep(50);
    return( 0 );
}

int create_bitmaps( void )
{

    int size = config.size;
    scrat = create_bitmap(size, size);
    if( (baro = load_bitmap( "..\\Resources\\alti\\baro.bmp", baro )) == NULL ) return( -1 );
    if( (alt = load_bitmap( "..\\Resources\\alti\\alt.bmp", alt )) == NULL ) return( -1 );
    if( (feet = load_bitmap( "..\\Resources\\alti\\feet.bmp", feet )) == NULL ) return( -1 );
    if( (_1K = load_bitmap( "..\\Resources\\alti\\1K.bmp", _1K )) == NULL ) return( -1 );
    if( (_10K = load_bitmap( "..\\Resources\\alti\\10K.bmp", _10K )) == NULL ) return( -1 );
    if( (frame = load_bitmap( "..\\Resources\\alti\\frame2.bmp", frame )) == NULL ) return( -1 );
    return( 0 );
}

int act_on_keypress( void )
{
    RECT rect;
    if (GetWindowRect(win_get_window(), &rect))
    {
        config.x_pos = (int)rect.left;
        config.y_pos = (int)rect.top;
    }
    keypress = readkey();
    switch (keypress)
    {

    case 6778: //z
        baro_val+=0.02;
        break;

    case 6264: //x
        baro_val-=0.02;
        break;



    case 867: //c
        alt_val -= 100;
        break;


    case 5750: //v
        alt_val += 100;
        break;


    case 4465 : //q
        _1K_val+=1.0;
        break;

    case 353: //a
        _1K_val-=1.0;
        break;

    case 6007 : //w
        _10K_val+=1.0;
        break;

    case 1381: //e
        _10K_val-=1.0;
        break;


    case 2409:         //i pressed ( increase size )
        if ( config.size < 600 ) config.size+=1;
        destroy_bitmap( scrat );
        scrat = create_bitmap(config.size, config.size);
        SetWindowPos(win_get_window(),
                     HWND_TOPMOST,
                     config.x_pos,
                     config.y_pos,
                     config.size+6,
                     config.size+25,
                     SWP_NOZORDER);

        scale_factor = ((float)config.size / 400);
        break;

    case 1124:         //d pressed ( decrease size )
        if( config.size > 142 ) config.size-=1;
        destroy_bitmap( scrat );
        scrat = create_bitmap(config.size, config.size);
        SetWindowPos(win_get_window(),
                     0,
                     config.x_pos,
                     config.y_pos,
                     config.size+6,
                     config.size+25,
                     SWP_NOZORDER);
        scale_factor = ((float)config.size / 400);
        break;
    }
    return( 0 );
}
int close_program( void )
{
    allegro_exit();
    return( 0 );
}
void exit_hook( void )
{
    loop = 0;
}

void cb_on_data(int block_id, float *data)
{
    int i;

    switch(block_id)
    {
    case 7:
        baro_val = data[0];
        break;

    case 118:
        alt_ap = data[3];
        break;

    case 20:
        alt_val = data[5];
        break;

    }

}








