/*--------------------------------------------------------
  "THE BEER-WARE LICENSE" (Revision 42):
  Alex Kostyuk wrote this code. As long as you retain this 
  notice you can do whatever you want with this stuff. 
  If we meet some day, and you think this stuff is worth it, 
  you can buy me a beer in return.
----------------------------------------------------------*/ 

#include "includes.h"


#define FIFO_SIZE 50
float gs=0.0,is=0.0;

typedef struct
{
    int push_indx;
    int pop_indx;
    float buffer[100];
} FIFO;

float Rnm = 0;

float R_turn(float bank, float gs)
{
    float pi_grad = 3.14159 / 180.0;
    float v_ms = gs * 1852.0 / 3600.0;
    float R = (v_ms * v_ms)/(9.81*tan(pi_grad * bank));
    return R/1852.0;
}

static void fifo_init(FIFO *fifo)
{
    fifo->push_indx = 0;
    fifo->pop_indx = 0;
}

static int fifo_push(FIFO *fifo, float data,int fifo_size)
{
    int indx_tmp;
    indx_tmp = fifo->push_indx + 1;
    indx_tmp = indx_tmp % fifo_size;

    if (indx_tmp == fifo->pop_indx)
    {
        return 0;
    }
    fifo->buffer[fifo->push_indx] = data;
    fifo->push_indx = indx_tmp;
    return 1;
}

static int fifo_pop(FIFO *fifo, float *data,int fifo_size)
{
    int indx_tmp;
    if (fifo->push_indx == fifo->pop_indx)
    {
        return 0;
    }
    indx_tmp = fifo->pop_indx + 1;
    indx_tmp = indx_tmp % fifo_size;
    *data = fifo->buffer[fifo->pop_indx];
    fifo->pop_indx = indx_tmp;
    return 1;
}


float moving_average_filter(float cur)
{
    float pre;
    static float ref_sum = 0.0;
    static FIFO f;
    static int flag = 0;

    if(flag==0)
    {
        fifo_init(&f);
        flag = 1;
    }

    ref_sum += cur;
    if(!fifo_push(&f,cur, FIFO_SIZE))
    {
        fifo_pop(&f,&pre, FIFO_SIZE);
        fifo_push(&f,cur, FIFO_SIZE);
        ref_sum -= pre;
    }
    else
        return 0.0;
    return ref_sum/((FIFO_SIZE-1)*1.0);
}

float moving_average_filter_pitch(float cur)
{
    float pre;
    static float ref_sum = 0.0;
    static FIFO f;
    static int flag = 0;

    if(flag==0)
    {
        fifo_init(&f);
        flag = 1;
    }

    ref_sum += cur;
    if(!fifo_push(&f,cur, FIFO_SIZE))
    {
        fifo_pop(&f,&pre, FIFO_SIZE);
        fifo_push(&f,cur, FIFO_SIZE);
        ref_sum -= pre;
    }
    else
        return 0.0;
    return ref_sum/((FIFO_SIZE-1)*1.0);
}

float moving_average_filter_turn(float cur)
{
    float pre;
    static float ref_sum = 0.0;
    static FIFO f;
    static int flag = 0;

    if(flag==0)
    {
        fifo_init(&f);
        flag = 1;
    }

    ref_sum += cur;
    if(!fifo_push(&f,cur, FIFO_SIZE))
    {
        fifo_pop(&f,&pre, FIFO_SIZE);
        fifo_push(&f,cur, FIFO_SIZE);
        ref_sum -= pre;
    }
    else
        return 0.0;
    return ref_sum/((FIFO_SIZE-1)*1.0);
}



/*** Define Variables ***/
int message_wait_time = 5;
int loop = 1, keypress;
float scale_factor;
long knots =101;
float angle, knots_fp;
char buffer[100];
DWORD dwResult;
HDC hdc;
FILE *inifile;
BITMAP *scrat, *frame, *turn_speed_niddle,*metall_ball,*glass_tube;
PALETTE palette;


float bank_angle = 0;
float ball_position = 0;


int initialise_window( void );
int connect_to_sim( void );
int update_display( void );
int create_bitmaps( void );

int act_on_keypress( void );
int close_program( void );
void exit_hook (void );


int main(int argc, char *argv[] )
{

    STARTUPINFO cif;
    PROCESS_INFORMATION pi;
    load_config();






    if(0 == strcmp(config.console,"off") && argc == 1)
    {
        ZeroMemory(&cif,sizeof(STARTUPINFO));
        if(CreateProcess(NULL,"eup53.exe AS_DAEMON",
                         NULL,NULL,FALSE,CREATE_NO_WINDOW,NULL,NULL,&cif,&pi))

        {
            CloseHandle( pi.hProcess );
            exit(0);
        }
    }

    if(config.uplink_port)
        server_start(config.uplink_port);


    if(config.downlink_port)
    {
        udp_sender_start(config.downlink_port, config.downlink_ip);
    }


    initialise_window();


    if( create_bitmaps() == -1 )
    {
        textprintf_centre(screen, font, config.size / 2, config.size * 0.15, 65535,  "*** Error ***" );
        textprintf_centre(screen, font, config.size / 2, config.size * 0.40, 65535,  "No Graphics files" );
        textprintf_centre(screen, font, config.size / 2, config.size * 0.65, 65535,  "Press Key to Exit" );
        readkey();
        close_program();
        return( -1 );
    }

    update_display();

    while( loop == 1 )
    {

        while( keypressed() )
        {
            act_on_keypress();
            update_display();
        }
        clear_keybuf();
        update_display();
    }

    save_config();
    close_program();
    return (0);
}
END_OF_MAIN()

int initialise_window( void )
{
    hdc = GetDC(NULL);
    int screen_colour_depth = GetDeviceCaps (hdc, BITSPIXEL );

    if (allegro_init() != 0) return 1; //Initialise Allegro
    install_keyboard();

    set_color_depth(screen_colour_depth);


    if (set_gfx_mode(GFX_AUTODETECT_WINDOWED, 800, 600, 0, 0) != 0)
    {
        if (set_gfx_mode(GFX_SAFE, 800, 600, 0, 0) != 0)
        {
            set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
            allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
            return 1;
        }
    }

    SetWindowPos(win_get_window(), HWND_TOPMOST,
                 config.x_pos,
                 config.y_pos,
                 config.size+6,
                 config.size+25,
                 SWP_ASYNCWINDOWPOS);

    set_display_switch_mode(SWITCH_BACKGROUND);
    scale_factor = ((float)config.size / 400);

    set_window_close_hook(exit_hook);
    return( 0 );
}


int update_display( void )

{
    int size = config.size;


    pivot_scaled_sprite(scrat, glass_tube,
                        size/2,
                        236.0*scale_factor,
                        140,
                        40,
                        ftofix(0) , ftofix(scale_factor));

    pivot_scaled_sprite(scrat, metall_ball,
                        size / 2 - ((ball_position)*1.0)*scale_factor,
                        243.0*scale_factor,
                        26,
                        34,
                        0 , ftofix(scale_factor));

    pivot_scaled_sprite(scrat, frame, size / 2, (size / 2),200, 200,
                        ftofix(0) ,ftofix(scale_factor));


    pivot_scaled_sprite(scrat, turn_speed_niddle,
                        size / 2,
                        313.0*scale_factor,
                        28,
                        235,
                        ftofix(bank_angle*1.0-0.5) , ftofix(scale_factor));

    if(console_flag)
        printf("bank: %f   pitch: %f sideG: %f\n",
               bank_angle,
               ball_position
              );

    Rnm = R_turn(bank_angle,gs/1.852);
    if(Rnm < -0.00001) Rnm = Rnm * -1.0;

    if(Rnm <= 50.0)
        textprintf_ex(scrat, font, 5, 5, makecol(255, 255, 000),-1,
                      "R:%3.1fnm/Bnk:%3.0fdeg",Rnm,bank_angle
                     );
    else
        textprintf_ex(scrat, font, 5, 5, makecol(255, 255, 000),-1,
                      "R:--/Bnk:%3.0fdeg",bank_angle
                     );

    blit(scrat, screen, 0, 0, 0, 0, size, size);
    Sleep(20);
    return( 0 );
}

int create_bitmaps( void )
{
    int size = config.size;
    scrat = create_bitmap(size, size);
    if( (turn_speed_niddle = load_bitmap( "..\\Resources\\eup53\\niddle.bmp", turn_speed_niddle )) == NULL ) return( -1 );
    if( (frame = load_bitmap( "..\\Resources\\eup53\\plate.bmp", frame )) == NULL ) return( -1 );
    if( (metall_ball = load_bitmap( "..\\Resources\\eup53\\ball.bmp", metall_ball )) == NULL ) return( -1 );
    if( (glass_tube = load_bitmap( "..\\Resources\\eup53\\pipe.bmp", glass_tube )) == NULL ) return( -1 );
    return( 0 );
}

int act_on_keypress( void )
{
    RECT rect;
    if (GetWindowRect(win_get_window(), &rect))
    {
        config.x_pos = (int)rect.left;
        config.y_pos = (int)rect.top;
    }
    keypress = readkey();
    switch (keypress)
    {

    case 6778: //
        bank_angle -= 1;
        break;

    case 6264: //x
        bank_angle += 1;
        break;



    case 867: //c

        ball_position--;
        break;


    case 5750: //v

        ball_position++;
        break;


    case 4465 : //q


        break;

    case 353: //a


        break;

    case 6007 : //w

        break;

    case 1381: //e

        break;


    case 2409:         //i pressed ( increase size )
        if ( config.size < 600 ) config.size+=1;
        destroy_bitmap( scrat );
        scrat = create_bitmap(config.size, config.size);
        SetWindowPos(win_get_window(),
                     HWND_TOPMOST,
                     config.x_pos,
                     config.y_pos,
                     config.size+6,
                     config.size+25,
                     SWP_NOZORDER);

        scale_factor = ((float)config.size / 400);
        break;

    case 1124:         //d pressed ( decrease size )
        if( config.size > 142 ) config.size-=1;
        destroy_bitmap( scrat );
        scrat = create_bitmap(config.size, config.size);
        SetWindowPos(win_get_window(),
                     0,
                     config.x_pos,
                     config.y_pos,
                     config.size+6,
                     config.size+25,
                     SWP_NOZORDER);
        scale_factor = ((float)config.size / 400);
        break;
    }
    return( 0 );
}
int close_program( void )
{
    allegro_exit();
    return( 0 );
}
void exit_hook( void )
{
    loop = 0;
}

void cb_on_data(int block_id, float *data)
{
    int i;

    switch(block_id)
    {
    case 3:
        is = 1.852*data[0];
        gs = 1.852*data[2];
        break;


    case 4:
        ball_position = moving_average_filter(data[6]*1000.0);
        if(ball_position>90.0) ball_position = 90.0;
        if(ball_position<-90.0) ball_position = -90.0;
        break;

    case 7:

        break;

    case 118:

        break;

    case 20:

        break;

    case 99:

        break;


    case 100:


        break;

    case 16:
        bank_angle = moving_average_filter_turn(data[2]*720/3.14159);
        if(bank_angle>25.0)
            bank_angle = 25.0;
        if(bank_angle<-25.0)
            bank_angle = -25.0;
        break;


    case 17:
        bank_angle = data[1];//*33.0/45.0;
        break;

    case 101:



        break;




    }




}








